% fit explanatory models to Colorwheel 5 (= post-cue experiment)

clear variables

% clusterX = parcluster('local');  %retrieves local cluster profile
% clusterX.NumWorkers = 4;         %changes number of workers from 4 to x
% parpool(clusterX,4);             %creates parallel pool with x workers

condOption = 1; % 1: fit to all 3 cueing conditions jointly; 3: fit to 3 cueing conditions separately

model = 7;  %choice of MM model version
% 1 = Slot-Averaging (fit to cueing conditions separately), 2 = Resource (fit to cueing conditions separately)
% 3 = Slot-Averaging Spatial Encoding (fit to cueing conditions separately)
% 4 = Slot-Averaging (6 parms, assignment of slots to focused item), 5 = Resource 6 (6-parameter, assignment of resources to focused item),
% 6 = Slot-Averaging Spatial Encoding (7 parms, assignment of slots to focused items)
% 7 = IM; 8 = IM with s=20; 9 = IM-Capacity, 10 = IM-Phase-space model, 11 = IM-TBRS
% 12 = GIM, 13 = IMguess1, 14 = IMVP

if model < 4, condOption = 3; end %these models must be fit to each cueing condition separately!
selection = 0; %0: no selection (all data), 1: only no-cue data,
if condOption == 3, selection = 0; end

Npar = [2, 2, 3,   6, 6, 7,   6, 6, 7,    5, 6, 7,   6, 7];

data = textread('colorwheel5.dat');

% column definition
subjectIDCol = 1;
sessionCol = 2;
cueCol = 3;      % 1 is the condition with valid/invalid cue, 3 is the condition with neutral cue.
trialIndexCol = 5;
setsizeCol = 6;
cueConditionCol = 7;
cuedLocationCol = 8;
targetColorCol = 9;
targetLocationCol = 10;
responseColorCol = 25;
stimulusColorCol = [9 11 13 15 17 19 21 23];
stimulusLocationCol = [10 12 14 16 18 20 22 24];
id = data(:,subjectIDCol);
idList = unique(id);

Preds = [];  %matrix to extract predictions for all trials
Parameters = zeros(length(idList), condOption, Npar(model));
LL = zeros(length(idList), condOption);
Exitflag = zeros(length(idList), 1);
iterations = 1;  %10
maxSZ = 8;
cueIndex = [-1, 1, 0];    %codes in data set for neutral, valid, invalid

for pID = 1:length(idList),
    
    for cond = 1:condOption
        
        Data = struct();
        expParms = struct();
        if selection == 0, subdata = data(id == idList(pID), :); end
        if selection == 1, subdata = data(id == idList(pID) & data(:,cueCol)==3, :); end
        if condOption == 3, subdata = data(id == idList(pID) & data(:,cueConditionCol)==cueIndex(cond), :); end  %selects data according to cond index
        expParms.setsize = subdata(:, setsizeCol);
        cangle = subdata(:,stimulusColorCol);   %color angle: location of stimulus in color wheel
        pangle = subdata(:,stimulusLocationCol);  %position angle: location of stimulus in space
        Probed = subdata(:, targetLocationCol);
        Data.target = subdata(:, targetColorCol);          %correct response (color angle)
        Data.response = subdata(:,responseColorCol);
        Data.cuedItem = repmat(subdata(:, cuedLocationCol), 1, maxSZ) == pangle;
        expParms.cue = subdata(:, cueConditionCol);
        expParms.N = size(Data.response,1);
        
        %starting parameters
        if model == 1
            startParms = [3, 15];   %Slot-Averaging model: k, sigma
            expParms.ulim = [8, 90];
            expParms.urange = expParms.ulim;
        end
        if model == 2
            startParms = [10, 1];   % Resource model: Kappa@1, alpha
            expParms.ulim = [100, 5];
            expParms.urange = [60, 2];
        end
        if model == 3
            startParms = [3, 20, 1];   % K, Sigma-color, s-spatial
            expParms.ulim = [8, 90, 20];
            expParms.urange = [8, 90, 10];
        end
        if model == 4
            startParms = [3, 15, 0.3, 0.3, 0.3, 0.3];   %Slot-Averaging model fit jointly: k, sigma, number of slots assigned to cued object at setsizes 2, 4, 6 8
            expParms.ulim = [8, 90, 1, 1, 1, 1];
            expParms.urange = expParms.ulim;
        end
        if model == 5
            startParms = [10, 1, 0.3, 0.3, 0.3, 0.3];   % Resource model fit jointly: Kappa@1, alpha, R-focus@2, R-focus@4, R-focus@6, R-focus@8
            expParms.ulim = [100, 5, 1, 1, 1, 1];
            expParms.urange = [60, 2, 0.8, 0.8, 0.8, 0.8];
        end
        if model == 6
            startParms = [3, 15, 1, 0.3, 0.3, 0.3, 0.3];   %Slot-Averaging model spatial fit jointly: k, sigma, s-spatial, number of slots assigned to cued object at setsizes 2, 4, 6 8
            expParms.ulim = [8, 90, 20, 1, 1, 1, 1];
            expParms.urange = expParms.ulim;
        end        
        if ismember(model, [7, 8, 13])
            startParms = [.1, .1, 2, 4, 20, .5];  %B, A, s, kappa, kappafocus, Creduction
            expParms.ulim = [10, 10, 20, 100, 200, 1];  %B, A, s, kappa, kappafocus, Creduction
            expParms.urange = [1, 1, 20, 100, 100, 1];
        end
        if model == 9
            startParms = [.1, .1, 2, 4, 20, .5, 6];  %B, A, s, kappa, kappafocus, Creduction, K
            expParms.ulim = [10, 10, 20, 100, 200, 1, max(expParms.setsize)];  %B, A, s, kappa, kappafocus, Creduction, K
            expParms.urange = [1, 1, 20, 100, 100, 1, max(expParms.setsize)];
        end
        if model == 10
            startParms = [.1, 10, 4, 20, .5];  %B, s, kappa, kappafocus, Creduction
            expParms.ulim = [10, 100, 100, 200, 1];  %B, s, kappa, kappafocus, Creduction
            expParms.urange = [1, 10, 100, 100, 1];
        end
        if model == 11
            startParms = [.1, .1, 2, 4, 20, 10];  %B, A, s, kappa, kappafocus, D
            expParms.ulim = [10, 10, 20, 100, 200, 200];  %B, A, s, kappa, kappafocus, D
            expParms.urange = [1, 1, 20, 100, 200, 100];
        end
        if model == 12
            startParms = [.1, .1, 2, 4, 20, .5, 2];  %B, A, s, kappa, kappafocus, Creduction, sf
            expParms.ulim = [10, 10, 20, 100, 200, 1, 20];  %B, A, s, kappa, kappafocus, Creduction, sf
            expParms.urange = [1, 1, 20, 100, 100, 1, 10];
        end
        if model == 14
            startParms = [.1, .1, 2, 4, 20, .5, 10];  %B, A, s, kappa, kappafocus, Creduction, tau
            expParms.ulim = [10, 10, 20, 100, 200, 1, 50];  %B, A, s, kappa, kappafocus, Creduction, tau
            expParms.urange = [1, 1, 20, 100, 100, 1, 50];
        end
        
        
        npar = length(startParms);  %number of parameters
        expParms.llim = zeros(1, npar);
        if model == 8, expParms.llim(3) = expParms.ulim(3); end  %fixing s = 20 in IM
        
        %fixed parms and experimental variables
        expParms.B=.1;
        expParms.Ci = 1;
        expParms.constraint = '';
        expParms.distribution = 1;
        
        % Pre-processing of data
        Probed = repmat(Probed, 1, maxSZ);
        D1 = abs(pangle-Probed);  %distance of probed location to all locations
        D2 = abs((pangle-13)-Probed); %distance of probed location to all locations, setting 13 to 0 etc, to capture distance across the 13->1 boundary
        D3 = abs(pangle - (Probed-13)); %ditto for distance between large Probed and smaller pangle)
        Data.D = min(min(D1, D2), D3);   %keep smallest distance
        Data.D = 2*pi*Data.D./13;       %convert into radians
        
        Cued = subdata(:, cuedLocationCol);
        Cued = repmat(Cued, 1, maxSZ);
        D1 = abs(pangle-Cued);
        D2 = abs((pangle-13)-Cued);
        D3 = abs(pangle - (Cued-13));
        Data.Df = min(min(D1, D2), D3);
        Data.Df = 2*pi*Data.Df./13;   %convert into radians
        Data.Df(Cued==-1) = 1;
        
        Cangle = repmat(cangle, [1, 1, 360]);
        [a, b, Colors] = ndgrid(ones(1,expParms.N), ones(1, maxSZ), 1:360);  %Colors = N x setsize x [1:360]
        Dc1 = abs(Cangle-Colors);
        Dc2 = abs((Cangle-360)-Colors);
        Dc3 = abs(Cangle-(Colors-360));
        Dc = min(min(Dc1, Dc2), Dc3);   %distances of each item in each trial from the 360 colors in color wheel: matrix of N x setsize x 360
        Data.Dcang = 2*pi*Dc/360;  %convert into scale from 0 to 2pi (for 1:360), so that distances range from 0 to pi.
        
        Tangle = repmat(Data.target, [1, 360]);
        Dt1 = abs(Tangle - squeeze(Colors(:,1,:)));
        Dt2 = abs((Tangle-360) - squeeze(Colors(:,1,:)));
        Dt3 = abs(Tangle - (squeeze(Colors(:,1,:))-360));
        Dt = min(min(Dt1, Dt2), Dt3);
        Data.Dtang = 2*pi*Dt/360;  %distance of target item in each trial from the 360 colors in color wheel (transformed to radians)
        
        % phase distance for IM-Phase-space model
        if model == 10
            tD = [1 1 2 2 3 3 4 4];
            Data.PhD(1).phaseD = 0; % for setsize 1, phase distance for the only item (= the target) = 0
            for setsize = 2:max(expParms.setsize)
                phaseD = tD./setsize;
                phD = perms(phaseD(1:(setsize-1)));
                if sum(Data.D(:,1)) == 0  % just to make sure that the first item in the vector is alway the target,
                    phD = [zeros(size(phD,1), 1), phD]; %put zeros in front because phase distance to target is always 0
                else
                    x = Alarm;  % this will throw an error
                end
                Data.PhD(setsize).phaseD = phD;
            end
        end
        
        if model == 12
            for f = 1:8
                F = repmat(pangle(:,f), 1, maxSZ); %reference location F
                D1 = abs(pangle-F);  %distance of F location to all 8 locations
                D2 = abs((pangle-13)-F);
                D3 = abs(pangle - (F-13));
                ddd = min(min(D1, D2), D3);   %keep smallest distance
                Data.d(:,f,:) = 2*pi*ddd./13;   %convert to radians
            end
        end
                
        % coding for IM
        Data.pfenc = zeros(size(Data.cuedItem,1), 1); % probability that target is in focus at test, given it was in focus at encoding
        Data.pfenc(sum(Data.cuedItem,2) == 0) = 1;  %neutral condition: focus is assumed to stay on the item it was directed to at encoding
        Data.pfenc(Data.cuedItem(:,1) == 1) = 1;    %valid condition: focus is assumed to always be on target
        Data.pfenc(Data.cuedItem(:,2) == 1) = 0;    %invalid condition: focus is never on target
        Data.pfnenc = zeros(size(Data.cuedItem,1), 1); % probability that target is in focus at test, given it was NOT in focus at encoding
        Data.pfnenc(sum(Data.cuedItem,2) == 0) = 0;  %neutral condition: focus is assumed to stay on the item it was directed to at encoding, so if target was not in focus at encoding, it isn't at test
        Data.pfnenc(Data.cuedItem(:,1) == 1) = 1;    %valid condition: focus is assumed to always be on target
        Data.pfnenc(Data.cuedItem(:,2) == 1) = 0;    %invalid condition: focus is never on target
                
        if model == 1, disp('ID  Cond  LL/1000     K           Sigma'); end
        if model == 2, disp('ID  Cond  LL/1000     Kappa       alpha'); end
        if model == 3, disp('ID  Cond  LL/1000     K           Sigma      SigmaSpat'); end
        if model == 4, disp('ID  Cond  LL/1000     K           Sigma      Nslot2     Nslot4     Nslot6     Nslot8'); end
        if model == 5, disp('ID  Cond  LL/1000     Kappa       alpha      R@2        R@4        R@6        R@8'); end
        if model == 6, disp('ID  Cond  LL/1000     K           Sigma      s          Nslot2     Nslot4     Nslot6     Nslot8'); end        
        if model == 7, disp('ID  Cond  LL/1000     B           A         s          kappa      kappafocus Creduction'); end
        if model == 8, disp('ID  Cond  LL/1000     B           A         s          kappa      kappafocus Creduction'); end
        if model == 9, disp('ID  Cond  LL/1000     B          A         s          kappa      kappafocus Creduction  K'); end
        if model == 10, disp('ID  Cond  LL/1000     B          s          kappa      kappafocus Creduction'); end
        if model == 11, disp('ID  Cond  LL/1000     B           A         s          kappa      kappafocus Decayrate'); end
        if model == 12, disp('ID  Cond  LL/1000     B           A         s          kappa      kappafocus Creduction sf'); end
        if model == 13, disp('ID  Cond  LL/1000     G           A         s          kappa      kappafocus Creduction'); end
        if model == 14, disp('ID  Cond  LL/1000     B           A         s          kappa      kappafocus Creduction tau'); end
        
        if model == 1, Model = @SlotAvg2; end
        if model == 2, Model = @Resource2; end
        if model == 3, Model = @SlotAvgSpatialE; end
        if model == 4, Model = @SlotAvg6; end
        if model == 5, Model = @Resource6; end
        if model == 6, Model = @SlotAvgSpatialE7; end
        if model == 7 || model == 8, Model = @IM; end
        if model == 9, Model = @IMC; end
        if model == 10, Model = @IMPhase; end
        if model == 11, Model = @IMTBRS; end
        if model == 12, Model = @GIM; end
        if model == 13, Model = @IMguess1; end
        if model == 14, Model = @IMVP; end
        
        finalParmsMM = zeros(iterations, npar);
        minusMaxLMM = zeros(1,iterations);
        for iter = 1:iterations
            if iter > 1, startParms = expParms.llim + (expParms.urange-expParms.llim).*rand(1,npar); end
            [finalParmsMM(iter,:),minusMaxLMM(iter)] = wrapper4fmin(startParms,Data,expParms,Model);
            fprintf('%d   %d  %d   ', pID, cond, minusMaxLMM(iter)*0.001);
            fprintf('%.3f      ', finalParmsMM(iter,:));
            fprintf('\n');
        end
        if npar > 3,   %try once more with the best-fitting parameters so far as start values
            best = find(minusMaxLMM == min(minusMaxLMM));
            startParms = finalParmsMM(best(1),:);
            worst = find(minusMaxLMM == max(minusMaxLMM));    % replace worst by new one
            [finalParmsMM(worst(1),:),minusMaxLMM(worst(1))] = wrapper4fmin(startParms,Data,expParms,Model);
            fprintf('%d   %d  %d   ', pID, cond, minusMaxLMM(worst)*0.001);
            fprintf('%.3f      ', finalParmsMM(worst,:));
            fprintf('\n');
        end
        
        best = find(minusMaxLMM == min(minusMaxLMM));
        LL(pID, cond) = minusMaxLMM(best(1));
        Parameters(pID, cond, :) = finalParmsMM(best(1),:);
        
        %print best-fitting parameters parameters
        disp('Best:');
        fprintf('%d   %d  %d   ', pID, cond, LL(pID, cond)*0.001);
        fprintf('%.3f      ', Parameters(pID, cond, :));
        fprintf('\n\n');
        
        %extract predictions
        predictions = Model(Parameters(pID, cond, :), Data, expParms);
        preds = [subdata, predictions]; %set experimental parameters for the extracted trials in the first 18 columns
        Preds = [Preds; preds];
        
    end
    
end;

delete(gcp); %gets rid of current parallel pool

sumLL = sum(sum(LL))

%Parameters = squeeze(Parameters);
ParametersKappa2SD = Parameters;
if ismember(model, [2,5])
    ParametersKappa2SD(:,:,1) = (180/pi).*k2sd(Parameters(:,:,1));  %using k2sd function from Paul Bays in BaysMixtureModel toolbox
end
if ismember(model, [7:9, 11, 12, 13, 14])
    ParametersKappa2SD(:,:,4:5) = (180/pi).*k2sd(Parameters(:,:,4:5));  %using k2sd function from Paul Bays in BaysMixtureModel toolbox
end
if model == 10
    ParametersKappa2SD(:,:,3:4) = (180/pi).*k2sd(Parameters(:,:,3:4));  %using k2sd function from Paul Bays in BaysMixtureModel toolbox
end
if model > 6
    ParametersKappa2SD(:,1) = ParametersKappa2SD(:,1) * 360;        %multiply B with number of response options, because in the models B was not multiplied with 1/360 (as would be done for a uniform distribution over the 360 response options)
end
BFParmsbyCuecondMean = squeeze(mean(ParametersKappa2SD,1))
BFParmsbyCuecondSD = squeeze(std(ParametersKappa2SD,1))
BFParmsbyCuecondMedian = squeeze(median(ParametersKappa2SD,1))

if selection == 0
    modelname = {'SlotAvg2', 'Resource2', 'SlotAvgSpatE', 'SlotAvg6', 'Resource6', 'SlotAvgSpatE7', 'IM', 'IMs=20', ...
        'IMC', 'IMPhase', 'IMTBRS', 'GIM', 'IMguess1', 'IMVP'};
end
if selection == 1
    modelname = {'SlotAvg2nc', 'Resource2nc', 'SlotAvgSpatEnc', 'SlotAvg6nc', 'Resource6nc', 'SlotAvgSpatE7nc', 'IMnc', 'IMnc.s=20',...
        'IMCnc', 'IMPhasenc', 'IMTBRSnc', 'GIMnc', 'IMguess1nc', 'IMVPnc'};
end
saveFileName = sprintf('fittingResults%s', modelname{model});
saveFilePred = sprintf('%sPreds', modelname{model});
save(saveFileName)
save(saveFilePred, 'Preds');


