% mainline program to fit the Variable Precision model to ColorWheel1

clear variables

% clusterX = parcluster('local');  %retrieves local cluster profile
% clusterX.NumWorkers = 6;         %changes number of workers from 4 to x
% parpool(clusterX,6);             %creates parallel pool with x workers

model = 4;  %1 = VP fit to cueing conditions separately, 2 = VP fit to cueing conditions simultaneously, 3 = VP-Binding, 4 = VP-NT
Npars = [3 7 4  4];
iterations = 1;
fitting = 1; % 1 = Simplex, 2 = genetic algorithm, 3 = patternsearch


data = textread('colorwheel5.dat');

% column definition
subjectIDCol = 1;
sessionCol = 2;
condCol = 3;
trialIndexCol = 5;
setsizeCol = 6;
cueConditionCol = 7;
cuedLocationCol = 8;
targetColorCol = 9;
targetLocationCol = 10;
responseColorCol = 25;
stimulusColorCol = [9 11 13 15 17 19 21 23];
stimulusLocationCol = [10 12 14 16 18 20 22 24];

id = data(:,1);
idList = unique(id);
dtimeList = unique(data(:, 5));

cueOption = [-1, 1, 0];      % codes in data set for neutral, valid, invalid
if ismember(model, [2:3]), cueOption = -9; end  % fit conditions simultaneously

Preds = [];  %matrix to extract predictions for all trials

LL = zeros(length(idList), 3);
Nconds = length(cueOption);
Parameters = zeros(length(idList), Nconds, Npars(model));

for pID = 1:length(idList),
    for conditionIndex = 1:Nconds
        
        Data = struct();
        expParms = struct();
        expParms.fitting = fitting; 
        
        % mapping between J and kappa - nicked from R. van den Berg's code
        expParms.kappa_max      = 10000;
        expParms.kappa_map      = [linspace(0,10,250) linspace(10.001,expParms.kappa_max,250)];
        expParms.J_map          = expParms.kappa_map.*besseli(1,expParms.kappa_map,1)./besseli(0,expParms.kappa_map,1);
        
        validList = data(:, subjectIDCol) == idList(pID);
        if cueOption(1) ~= -9,
            validList = validList & (data(:, cueConditionCol) == (cueOption(conditionIndex)));
        end;
        expParms.setsize = data(validList,setsizeCol);
        
        response = data(validList,responseColorCol);
        expParms.N = size(response,1);
        target = data(validList, targetColorCol);
        
        Data.cangle = data(validList,stimulusColorCol);   %color angle: location of stimulus in color wheel
        Data.pangle = data(validList,stimulusLocationCol);  %position angle: location of stimulus in space
        
        for item = 1:8
            Dev1 = abs(Data.cangle(:,item)-response);   %absolute deviation of response from correct color (= color of first item)
            Dev2 = abs(Data.cangle(:,item)-(response-360));
            Dev3 = abs((Data.cangle(:,item)-360)-response);
            Dev = min(min(Dev1,Dev2),Dev3);
            Data.Dev(:,item) = Dev;
        end
        
        %starting parameters
        if model == 1, startParms = [15, 2, 1]; end % Variable-Precision Model: J1, tau, alpha
        if model == 2, startParms = [15, .5, .5, .5, .5, 2, 1]; end %VP fitting conditions simultaneously: J1, resources to cued item @2, @4, @6, @8, tau, alpha
        if model == 3, startParms = [15, 2, 1, 0.5]; end % Variable-Precision model for bindings: J1f, tau, J1s scaling factor, resource to cued item
		if model == 4, startParms = [15, 2, 1, 0.1]; end % Variable-Precision Model with NT responses: J1, tau, alpha, gamma
        
        npar = length(startParms);  %number of parameters
        
        % Extraction of data, conversion into radians
        Data.target = 2*pi*target./360;
        Data.response = 2*pi*response./360;
        Data.responsegrad = response;
        dtarget = repmat(1:360, expParms.N, 1) - repmat(target, 1, 360);
        dtarget(dtarget < -180) = dtarget(dtarget < -180) + 360;
        dtarget(dtarget > 180) = dtarget(dtarget > 180) - 360;
        Data.dtarget = 2*pi*dtarget/360;  %360 response options, expressed as deviation from the target, transformed into radians
        
        Data.cueCondition = data(validList, cueConditionCol);
        
        if model == 1, expParms.llim = zeros(1, npar); expParms.ulim = [200, 100, 5]; expParms.urange = [100, 50, 2]; end
        if model == 2, expParms.llim = zeros(1, npar); expParms.ulim = [200, 1, 1, 1, 1, 100, 5]; expParms.urange = [100, 1, 1, 1, 1, 50, 2]; end
        if model == 3, expParms.llim = zeros(1, npar); expParms.ulim = [200, 100, 50, 1]; expParms.urange = [100, 500, 50, 1];  end
   		if model == 4, expParms.llim = zeros(1, npar); expParms.ulim = [200, 100, 5, 1]; expParms.urange = [100, 50, 2, 0.7]; end       
        
        if model == 1, disp('    ID       LL/1000      J1       tau      alpha'); end
        if model == 2, disp('    ID       LL/1000      J1       pR@2     pR@4     pR@6      pR@8        tau      alpha'); end
        if model == 3, disp('    ID       LL/1000      J1f      tauF     J1s factor w'); end
 		if model == 4, disp('    ID       LL/1000      J1       tau      alpha    gamma'); end

        
        finalParmsMM = zeros(iterations, npar);
        minusMaxLMM = zeros(1,iterations);
        
        for iter = 1:iterations
            
            if iter > 1, startParms = expParms.llim + (expParms.urange-expParms.llim).*rand(1,npar); end
            
            if model == 1, Model = @VPDT; end
            if model == 2, Model = @VP7; end
            if model == 3, Model = @VPB; end
            if model == 4, Model = @VPNT; end
            
            expParms.mode = 1; %fitting: model returns LL of response only
            [finalParmsMM(iter,:),minusMaxLMM(iter)] = wrapper4VPfmin(startParms,Data,expParms,Model);
            
            %print best-fitting parameters parameters
            disp([pID, minusMaxLMM(iter)*0.001  finalParmsMM(iter,:)]);
            
        end
        
        best = find(minusMaxLMM == min(minusMaxLMM));
        Parameters(pID, conditionIndex, :) = finalParmsMM(best(1),:);
        
        %extract predictions
        expParms.mode = 2;
        outcome = Model(finalParmsMM(best(1),:), Data, expParms);
        predictions = outcome.Pred;
        LL(pID,conditionIndex) = outcome.likelihood;
        
        %print best-fitting parameters parameters
        disp('Best:');
        disp([pID, LL(pID,conditionIndex)*0.001  finalParmsMM(best(1), :)]);
        
        preds = [data(validList, 1:end), predictions]; %set experimental parameters for the extracted trials in the first 18 columns
        Preds = [Preds; preds];
        
    end;
end;

sumLL = sum(sum(LL))

BFParmsbyPrestime = squeeze(mean(Parameters,1))
BFParmsbyPrestimeMedian = squeeze(median(Parameters,1))
BFParms = squeeze(mean(mean(Parameters)))'


if model == 1,
    save fittingResultsVPDT
    save VPDTPreds Preds
end
if model == 2,
    save fittingResultsVP7
    save VP7Preds Preds
end
if model == 3,
    save fittingResultsVPB
    save VPBPreds Preds
end
if model == 4,
    save fittingResultsVPNT
    save VPNTPreds Preds
end
