% Compute predictions for Generalized IM for Color Wheel Post-Cue: 
% Focus contains a blend of the primary focused item and its neighbors,
% weighted by an exponential generalization gradient. That blend is added
% to E in proportion to the match between the cued location and the
% strength of that spatial location in the focus

function Pred = GIM(modelParms,Data,expParms)
B = modelParms(1);
A = modelParms(2); 
s = modelParms(3);
kappa = modelParms(4);
kappafocus = modelParms(5);
Creduction = modelParms(6);
sf = modelParms(7); 

cueing = exp(-s*Data.D);      %probability of retrieval for each memory object, as a function of spatial distanc
positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize
cueing(nonExistingItems) = 0; %zero probability for objects that don't exist
Pchoose = VonMisesN(Data.Dcang, 0, kappa, 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved 
Pchoosefocus = squeeze(VonMisesN(Data.Dcang, 0, kappafocus, 3)); %for each trial and each item j in that trial, probability of choosing color i based on activation of item j (j is focused) 

Evidence = zeros(expParms.N, 360); 
for trial = 1:expParms.N   
    sz = expParms.setsize(trial);
    Eb = sz*B/360; 
    Ea = A * squeeze(sum(Pchoose(trial,1:sz,:),2))'; 
    Ec = squeeze(sum(bsxfun(@times, cueing(trial,:), Pchoose(trial,:,:))))';    
    focuscenter = find(Data.cuedItem(trial,:) == 1);
    if isempty(focuscenter), focuscenter = 1:sz; end     %when no item is focused (neutral condition)
    evidence = zeros(length(focuscenter), 360);
    for f = focuscenter   %f is the index of the location at whith the focus peaks
       wfocus = squeeze(exp(-sf*Data.d(trial, f, :)));  %focus weight
       Efocus = bsxfun(@times, wfocus(1:sz), squeeze(Pchoosefocus(trial, 1:sz, :)));  %focus activation multiplied with focus gradient
       if sz == 1, Efocus = Efocus'; end
       evidence(f,:) = (1-wfocus(1)) * (Eb + Ea + Ec) + wfocus(1) * (Creduction * (Eb + Ea) + Ec + sum(Efocus,1));  % Ea is the blended color activation in focus; it is used in proportion to the match between the cued location (=1) and the blended location info in the focus (= pfocus)
    end
    Evidence(trial,:) = mean(evidence,1);  %average over all possible items being focused
end
Pred = bsxfun(@rdivide, Evidence, sum(Evidence, 2));  %normalize by dividing by the sum across all 360 colors
help = 1;

