% Compute predictions for IM for Color Wheel Post-Cue: 

function Pred = IM(modelParms,Data,expParms)
C = 1;
B = modelParms(1);
A = modelParms(2);
s = modelParms(3);
kappa = modelParms(4);
kappafocus = modelParms(5);
Creduction = modelParms(6);

cueing = exp(-s*Data.D);      %strength of cueing each memory object, as a function of spatial distanc
positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize
cueing(nonExistingItems) = 0; %zero probability for objects that don't exist
Pfe = 1./expParms.setsize; %probability of focusing the target at encoding
pfocused = Pfe.*Data.pfenc + (1-Pfe).*Data.pfnenc; %probability of focusing the target at retrieval

Pchoose = VonMisesN(Data.Dcang, 0, kappa, 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved (and is not focused)
Pchoose(repmat(nonExistingItems, [1 1 360])) = 0; % this is necessery for A component
Pchoosefocus = squeeze(VonMisesN(Data.Dcang(:,1,:), 0, kappafocus, 3)); %for each trial, probability of choosing color i, given the target is focused (high-precision representation exists only for one item, does not spill over to neighbors!)

Evidence = zeros(expParms.N, 360); 
for trial = 1:expParms.N
    Eb = B*(1/360)*expParms.setsize(trial); 
    Ea = squeeze(sum(Pchoose(trial,:,:),2))'; 
    Ec = squeeze(sum(bsxfun(@times, cueing(trial,:), Pchoose(trial,:,:))))'; 
    Evidence(trial,:) = (1-pfocused(trial)).*(Eb + A*Ea + C*Ec) ...
        + Pfe(trial)*Data.pfenc(trial) * (Creduction*Eb + Creduction*A*Ea + C*(Ec + Pchoosefocus(trial,:)) ) ...
        + (1-Pfe(trial))*Data.pfnenc(trial) * (Creduction*Eb + Creduction*A*Ea + C*Ec); 
end
Pred = bsxfun(@rdivide, Evidence, sum(Evidence, 2));  %normalize by dividing by the sum across all 360 colors
help = 1;

