% Compute predictions for IM-Capacity for Color Wheel Post-Cue

function Pred = IMC(modelParms,Data,expParms)
C = 1;
B = modelParms(1);
A = modelParms(2);
s = modelParms(3);
kappa = modelParms(4);
kappafocus = modelParms(5);
Creduction = modelParms(6);
K = modelParms(7);

cueing = exp(-s*Data.D);      %strenght of cueing for each memory object, as a function of spatial distance
positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize
cueing(nonExistingItems) = 0; %zero probability for objects that don't exist
Pfe = min(1, 1./min(K,expParms.setsize));  %probability of focusing the target at encoding
pfocused = Pfe.*Data.pfenc + (1-Pfe).*Data.pfnenc; %probability of focusing the target at retrieval = P(T@Enc) * P(T@R|T@Enc) + P(Tenc) + P(T@R|T@Enc)

Pchoose = VonMisesN(Data.Dcang, 0, kappa, 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved (and is not focused)
Pchoose(repmat(nonExistingItems, [1 1 360])) = 0; % this is necessery for A component
Pchoosefocus = squeeze(VonMisesN(Data.Dcang(:,1,:), 0, kappafocus, 3)); %for each trial, probability of choosing color i, given the target is focused (high-precision representation exists only for one item, does not spill over to neighbors!)

Pred = zeros(expParms.N, 360);
for trial = 1:expParms.N
    
    Eb = B*(1/360)*expParms.setsize(trial);
    
    % going through all possible combinations of assignment of slots to objects for determining the
    % probabilities of retrieving each object
    Nslots = [floor(K), ceil(K)];     %possible numbers of slots considered
    Pslots = [ceil(K)-K, K-floor(K)]; %probabilities of the 2 slot values considered
    if floor(K) == ceil(K), Nslots = K; Pslots = 1; end  %in case of integer K values!
    Condpred = zeros(length(Nslots), 360);
    for slots = 1:length(Nslots)
        if Nslots(slots) == 0,
            condpred = ones(1,360)./360;  %if there is no slot: uniform distribution of evidence
        else
            if Nslots(slots) >= expParms.setsize(trial)
                combinations = 1:expParms.setsize(trial);
            else
                combinations = nchoosek(1:expParms.setsize(trial),Nslots(slots));
            end
            condpred = zeros(size(combinations,1), 360); %conditional prediction for a given combination
            for c = 1:size(combinations,1)
                pf = ismember(1,combinations)*pfocused(trial); %probability that the target is in the focus - for valid retro-cues, this is 1 if ismember(1,combination); for invalid retro-cues pfocused = 0 anyway
                Ea = squeeze(sum(Pchoose(trial,combinations(c,:),:),2))';  %selecting only the items included in the currently considered combination
                Ec = squeeze( sum( bsxfun(@times, cueing(trial,combinations(c,:)), Pchoose(trial,combinations(c,:),:)), 2)  )';                
                evidence = (1-pf).*(Eb + A*Ea + C*Ec) + pf.*(Creduction*Eb + Creduction*A*Ea + C*(Ec + Pchoosefocus(trial,:)));
                condpred(c,:) = evidence./sum(evidence);
            end
        end
        Condpred(slots, :) = mean(condpred,1); %unweighted average, because all combinations are equally likely
    end
    Pred(trial,:) = Pslots * Condpred; %sum across considered numbers of slots, each weighted by the probability of having that number of slots
end
help = 1;



