% Compute predictions for Phase-space model for Color Wheel retro-cue experiment: 

function Pred = IMPhase(modelParms,Data,expParms)
C = 1;
B = modelParms(1);
s = modelParms(2);
kappa = modelParms(3);
kappafocus = modelParms(4);
Creduction = modelParms(5);

positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize

pfocused = Data.cuedItem + repmat((1-sum(Data.cuedItem,2)) .* (1./expParms.setsize), 1, max(expParms.setsize)); 
pfocused(nonExistingItems) = 0; %zero probability for objects that don't exist

Pchoose = VonMisesN(Data.Dcang, 0, kappa, 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved (and is not focused)
Pchoose(repmat(nonExistingItems, [1 1 360])) = 0; % this is necessery for A component
Pchoosefocus = squeeze(VonMisesN(Data.Dcang(:,1,:), 0, kappafocus, 3)); %for each trial, probability of choosing color i, given the target is focused (high-precision representation exists only for one item, does not spill over to neighbors!)


Evidence = zeros(expParms.N, 360); 
for trial = 1:expParms.N
    pretrieve = zeros(1,max(expParms.setsize));
    pretrieve(1:expParms.setsize(trial)) = mean( exp(-s*Data.PhD(expParms.setsize(trial)).phaseD), 1 );  %average over all permutations (= rows) 
    pretrieve = pretrieve./sum(pretrieve);  %normalize (not necessariy, but to keep consistency with IM)
    Eb = B*(1/360)*expParms.setsize(trial); 
    Ec = squeeze(sum(bsxfun(@times, pretrieve, Pchoose(trial,:,:))))'; 
    Evidence(trial,:) = (1-pfocused(trial)).*(Eb + C*Ec) + pfocused(trial).*(Creduction*Eb + C*(Ec + Pchoosefocus(trial,:)));
end
Pred = bsxfun(@rdivide, Evidence, sum(Evidence, 2));  %normalize by dividing by the sum across all 360 colors
help = 1;

