% Compute predictions for IM-TBRS for Color Wheel 5

function Pred = IMTBRS(modelParms,Data,expParms)
B = modelParms(1);
A = modelParms(2);
s = modelParms(3);
kappa = modelParms(4);
kappafocus = modelParms(5);
D = modelParms(6);
r = 0.05;  %needs to be fixed because it cannot be identified independently of D

Cnocue = exp(-D*(0:7)*r); %levels of C for the items rehearsed 7 to 0 steps (of length r) back in the past, at the time of test
Ccue = exp(-D*0.3); %all not-cued items are never refreshed, so they decay for the entire post-cue retention interval

cueing = exp(-s*Data.D);      %strength of cueing each memory object, as a function of spatial distanc
positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize
cueing(nonExistingItems) = 0; %zero cueing strength for objects that don't exist
pfocused = Data.cuedItem(:,1) + (1-sum(Data.cuedItem,2)) .* (1./expParms.setsize); % P(focused) = 1 for cued item, 0 for not-cued items, 1/setsize in neutral condition, when sum(CuedItem) = 0
%if one item was cued, its probability of being in the focus = 1, and 0 for
%all other items; if no item was cued, so that sum(Data.cuedItem) = 0, then
%all items receive 1/setsize probability of being in the focus

Pchoose = VonMisesN(Data.Dcang, 0, kappa, 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved (and is not focused)
Pchoose(repmat(nonExistingItems, [1 1 360])) = 0; % this is necessery for A component
Pchoosefocus = squeeze(VonMisesN(Data.Dcang(:,1,:), 0, kappafocus, 3)); %for each trial, probability of choosing color i, given the target is focused (high-precision representation exists only for one item, does not spill over to neighbors!)

Pred = zeros(expParms.N, 360); 
for trial = 1:expParms.N
    C = sum(Data.cuedItem(trial,:)) * Ccue + (1-sum(Data.cuedItem(trial,:))) * Cnocue; 
    Eb = B*(1/360)*expParms.setsize(trial); 
    Ea = A*squeeze(sum(Pchoose(trial,:,:),2))'; 
    Efocus = Eb + Ea + squeeze(sum(bsxfun(@times, cueing(trial,:), Pchoose(trial,:,:))))'; 
    predfocus = Efocus./sum(Efocus);
    if expParms.setsize(trial) > 1
        Enofocus = Eb + repmat(Ea, expParms.setsize(trial)-1, 1) + C(2:expParms.setsize(trial))' * ...
            squeeze(sum(bsxfun(@times, cueing(trial,:), Pchoose(trial,:,:))))';
        prednofocus = bsxfun(@rdivide, Enofocus, sum(Enofocus, 2));
        Pred(trial,:) = pfocused(trial) * predfocus + (1-pfocused(trial)) * mean(prednofocus, 1);
    else
        Evidence = Eb + Ea + squeeze(Pchoose(trial, 1, :))' + Pchoosefocus(trial,:);  %if setsize == 1, item is always in the focus (and stays there!)
        Pred(trial,:) = Evidence./sum(Evidence); 
    end

end
help = 1;

