% Variable-Precision version of IM

function Pred = IMVP(modelParms,Data,expParms)

C = 1;
B = modelParms(1);
A = modelParms(2);
s = modelParms(3);
Mkappa = modelParms(4);
kappafocus = modelParms(5);
Creduction = modelParms(6);
tau = modelParms(7);

nsim = 5000;
kappa = gamrnd(repmat(Mkappa, nsim, 1)/tau,tau);  % generate nsim kappas for simulation
x = 2*pi*(0:359)./360;  %values for which von-Mises is evaluated: first value = 0, so this is where the peak is!
vM = VonMisesN(repmat(x, nsim, 1), 0, repmat(kappa, 1, 360));
MvM = mean(vM);

cueing = exp(-s*Data.D);      %probability of retrieval for each memory object, as a function of spatial distanc
positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize
cueing(nonExistingItems) = 0; %zero probability for objects that don't exist
%cueing = cueing./repmat(sum(cueing,2), 1, max(expParms.setsize)); %need to divide by sum to get probabilities summing to 1 over the n objects
Pfe = 1./expParms.setsize; %probability of focusing the target at encoding
pfocused = Pfe.*Data.pfenc + (1-Pfe).*Data.pfnenc; %probability of focusing the target at retrieval

Pchoosefocus = squeeze(VonMisesN(Data.Dcang(:,1,:), 0, kappafocus, 3)); %for each trial, probability of choosing color i, given the target is focused (high-precision representation exists only for one item, does not spill over to neighbors!)
Evidence = zeros(expParms.N, 360);
for trial = 1:expParms.N
    Pchoose = zeros(max(expParms.setsize), 360);
    for item = 1:expParms.setsize(trial)
        itemIdx = find(Data.Dcang(trial,item,:) == 0); %find original item's number on the 1:360 scale
        Pchoose(item,:) = circshift(MvM, [0, itemIdx-1]);  %shift mean von-Mises distribution so that its peak is on the item
    end
    Eb = (B/360)*expParms.setsize(trial);
    Ea = sum(Pchoose);
    Ec = squeeze(sum( bsxfun(@times, cueing(trial,:), Pchoose'),2))';
    Evidence(trial,:) = (1-pfocused(trial)).*(Eb + A*Ea + C*Ec) ...
        + Pfe(trial)*Data.pfenc(trial) * (Creduction*Eb + Creduction*A*Ea + C*(Ec + Pchoosefocus(trial,:)) ) ...
        + (1-Pfe(trial))*Data.pfnenc(trial) * (Creduction*Eb + Creduction*A*Ea + C*Ec);
end
Pred = bsxfun(@rdivide, Evidence, sum(Evidence, 2));  %normalize by dividing by the sum across all 360 colors
help = 1;



