% Compute predictions for IM-guess for Color Wheel 5: 

function Pred = IMguess1(modelParms,Data,expParms)
C = 1;
G = modelParms(1);
A = modelParms(2);
s = modelParms(3);
kappa = modelParms(4);
kappafocus = modelParms(5);
Creduction = modelParms(6);

radbins = pi*(1:360)./180;
cueing = exp(-s*Data.D);      %strength of cueing each memory object, as a function of spatial distanc
positionInd = repmat(1:max(expParms.setsize), length(expParms.setsize), 1);
nonExistingItems = positionInd > repmat(expParms.setsize, 1, max(expParms.setsize));  %index into items that don't exist for a given setsize
cueing(nonExistingItems) = 0; %zero probability for objects that don't exist
Pfe = 1./expParms.setsize; %probability of focusing the target at encoding
pfocused = Pfe.*Data.pfenc + (1-Pfe).*Data.pfnenc; %probability of focusing the target at retrieval

Pchoose = VonMisesN(Data.Dcang, 0, kappa, 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved (and is not focused)
Pchoose(repmat(nonExistingItems, [1 1 360])) = 0; % this is necessery for A component
Pchoosefocus = squeeze(VonMisesN(Data.Dcang(:,1,:), 0, kappafocus, 3)); %for each trial, probability of choosing color i, given the target is focused (high-precision representation exists only for one item, does not spill over to neighbors!)

Pred = zeros(expParms.N, 360); 

for trial = 1:expParms.N

    Ea = squeeze(sum(Pchoose(trial,:,:),2))'; 
    Ec = squeeze(sum(bsxfun(@times, cueing(trial,:), Pchoose(trial,:,:))))'; 
    
    Evidence_nonfocus = A*Ea + C*Ec;
	Evidence_focus_enc = Creduction*A*Ea + C*(Ec + Pchoosefocus(trial,:));
    Evidence_focus_nenc = Creduction*A*Ea + C*Ec;
    pGuess_nonfocus = G * circ_var(radbins', Evidence_nonfocus');
    pGuess_focus_enc = G * circ_var(radbins', Evidence_focus_enc');   
    pGuess_focus_nenc = G * circ_var(radbins', Evidence_focus_nenc');   
    E_nf = Evidence_nonfocus/sum(Evidence_nonfocus);   %normalize by dividing by the sum across all 360 colors
    E_fe = Evidence_focus_enc/sum(Evidence_focus_enc);
    E_fne = Evidence_focus_nenc/sum(Evidence_focus_nenc);
    P_nf = (1-pGuess_nonfocus) * E_nf + pGuess_nonfocus * 1/360;  %mixture predictions for the case of a non-focused target
    P_fe = (1-pGuess_focus_enc) * E_fe + pGuess_focus_enc * 1/360; 
    P_fne = (1-pGuess_focus_nenc) * E_fne + pGuess_focus_nenc * 1/360; 
    
    Pred(trial,:) = (1-pfocused(trial)).* P_nf ...
        + Pfe(trial)*Data.pfenc(trial) * P_fe ...
        + (1-Pfe(trial))*Data.pfnenc(trial) * P_fne; 
end
help = 1;

