% compute predictions for 6-parameter original Resource model for Color Wheel
% To be applied to all cueing conditions jointly: Cued item receives extra
% share of resources


function Pred = Resource6(modelParms,Data,expParms)

cueOption = [-1, 1, 0];    %codes in data set for neutral, valid, invalid   

Kappa = modelParms(1);  %precision at setsize 1
Setsize = unique(expParms.setsize)';
npar = length(modelParms);
Rfocus(Setsize(2:end)) = modelParms(npar-3:npar); %proportion of resource additionally assigned to focused item at set sizes 2, 4, 6, 8
rshare = zeros(size(expParms.setsize)); 
Rshare = zeros(max(Setsize),3); 
for sz = Setsize
    Rshare(sz,1) = 1/sz;   %neutral
    Rshare(sz,2) = Rfocus(sz) + (1-Rfocus(sz))*(1/sz);   % valid cue condition
    Rshare(sz,3) = (1-Rfocus(sz))*(1/sz);                % invalid cue condition
    for cuecond = 1:3
       rshare(expParms.setsize == sz & expParms.cue == cueOption(cuecond)) = Rshare(sz, cuecond);   % assign Rshare to trials, forming a vector across all trials
    end
end

alpha = modelParms(2);  %exponent for setsize in kappa ~ 1/N^alpha
kappa = repmat(Kappa.*(rshare.^alpha), 1, 360);  %replacing 1/N by Rshare, and multiplying with Kappa as the precision for setsize 1

pchoose = VonMises(squeeze(Data.Dcang(:,1,:)), 0, kappa); %for each trial, for the first item in that trial (= target), probability of choosing color i
Pred = pchoose./repmat(sum(pchoose,2), 1, 360);  %normalize
help = 1; 
