function Pred = SlotAvg6(modelParms, Data, expParms)
% Slot-and-averaging model of Zhang & Luck. By HY Lin, modified by KO
% (August 2012) - this version is NOT identical with the one for Colorwheel
% (1), because it was adapted to the structure of the GMM models (Jan 2013)

cueOption = [-1, 1, 0];    %codes in data set for neutral, valid, invalid   

k = modelParms(1);   %number of slots
originalSigma = 2*pi*modelParms(2)/360;   %standard deviation of each slot, converted from degrees into radians
Setsize = unique(expParms.setsize)';
slots4cued = zeros(1,max(Setsize));
slots4cued(Setsize(2:end)) = modelParms(3:6); %proportion of slots assigned to cued item for set sizes 2, 4, 6, 8
slots4cued = k*slots4cued;                    %number of slots assigned to cued item... 
sigmaList = ones(3,max(Setsize))*originalSigma; 
pretrieve = zeros(3,max(Setsize));

%determine for each set size how many slots, on average, are assigned to an
%item, given a value of k, and compute SD (sigma) from that

for sz = Setsize
    % neutral condition
    slotproportion(1) = k/sz;  %proportion of slots available for each item (e.g., if k = 5, and sz = 2, slotproportion = 2.5)
    pretrieve(1, sz) = min(1, k./sz); 
    % valid condition
    if sz == 1, 
        slotproportion(2) = k; 
        pretrieve(2,sz) = min(1, k/sz); 
    else
        slotproportion(2) = slots4cued(sz); 
        pretrieve(2, sz) = min(1, slots4cued(sz));
    end
    % invalid condition
    if sz == 1,   %irrelevant, this does not happen
        slotproportion(3) = 1; 
        pretrieve(3,sz) = 0;
    else
        slotproportion(3) = max(0, k-slots4cued(sz))/sz; 
        pretrieve(3, sz) = min(1, max(0, (k-slots4cued(sz))/sz));
    end
    for cuecond = 1:3
        if slotproportion(cuecond) > 1
            wholeslots = floor(slotproportion(cuecond));  %number of whole slots assigned to every item (in the example, every item gets at least 2 slots)
            rest = slotproportion(cuecond)-wholeslots;    %as yet unassigned part of slotproportion (e.g., 0.5), equals the proportion of items that receive one extra slot
            sigmaList(cuecond,sz) = (1-rest)*originalSigma./sqrt(wholeslots) + rest*originalSigma/sqrt(wholeslots+1);  %a proportion (1-rest) of items receive wholeslots, a proportion rest of all items receives one additional slot
        end
    end
end

Pretrieve = zeros(size(expParms.setsize));
Kappa = zeros(size(expParms.setsize));
for sz = Setsize
    for cuecond = 1:3
        Pretrieve(expParms.setsize==sz & expParms.cue==cueOption(cuecond)) = pretrieve(cuecond, sz);
        Kappa(expParms.setsize==sz & expParms.cue==cueOption(cuecond)) = sd2k(sigmaList(cuecond, sz));
    end
end
kappa = repmat(Kappa, [1, 360]); 
pchoose = VonMisesN(squeeze(Data.Dcang(:,1,:)), 0, kappa); %for each trial, for the first item in that trial (= target), probability of choosing color i
Pchoose = pchoose./repmat(sum(pchoose,2), 1, 360);  %normalize
Pred = repmat(Pretrieve, 1, 360).*Pchoose + repmat(1-Pretrieve, 1, 360).*(1/360);   
help = 1; 
