% compute predictions for Variable Precision Model for Color Wheel

function output = VPDT(modelParms,Data,expParms)

J1bar  = modelParms(1);    % mean J at setsize 1
tau = modelParms(2);       % scale of Gamma distribution for Js
alpha = modelParms(3);     % exponent of the power function for Js over setsize
eval(expParms.constraint);

nsim = 5000;  % number of simulations
stream = RandStream('mt19937ar','Seed',28);
RandStream.setGlobalStream(stream);
szList = unique(expParms.setsize)';

% nicked from R. van den Berg's code
Jbar = J1bar./(szList.^alpha); %vector of mean J's, one for each set size
Jbar = repmat(Jbar, nsim, 1); %replicates vector of mean J's for each simulation
J = gamrnd(Jbar/tau,tau);
J = min(J,max(expParms.J_map));
kappa = interp1(expParms.J_map,expParms.kappa_map,J);

if expParms.mode == 1
	vPDFSz = zeros(length(szList), 360);
    for sim = 1:nsim
		for szInd = 1:length(szList),
			vPDF = VonMisesN([0:359]*2*pi/360, 0, kappa(sim, szInd));
			if sum(vPDF) == 0 || any(isnan(vPDF)), vPDF(:) = eps; vPDF(1) = 1; end %when kappa > 700, vonMises evaluates all to 0 -> set vPDF to 1 at the target location
			vPDFSz(szInd, :) = vPDFSz(szInd, :) + vPDF./sum(vPDF);
		end
    end
	vPDFSz = vPDFSz ./ nsim;
	
    likelihood = zeros(expParms.N, 1); 
	for trial = 1:expParms.N
		szInd = find(szList == expParms.setsize(trial), 1);
		likelihood(trial) = vPDFSz(szInd, Data.Dev(trial)+1);
	end
	output = likelihood;
end

if expParms.mode == 2  %calculate predicted probability of choosing for all 360 colors
	vPDFSz = zeros(length(szList), 360); % PDF for each set size.
    for sim = 1:nsim 
		for szInd = 1:length(szList),
			vPDF = VonMisesN([0:359]*2*pi/360, 0, kappa(sim, szInd));
			if sum(vPDF) == 0 || any(isnan(vPDF)), vPDF(:) = eps; vPDF(1) = 1; end %when kappa > 700, vonMises evaluates all to 0 -> set vPDF to 1 at the target location
			vPDFSz(szInd, :) = vPDFSz(szInd, :) + vPDF./sum(vPDF);
		end
    end
	vPDFSz = vPDFSz ./ nsim;
	
    Pchoose = zeros(expParms.N, 360);
	for trial = 1:expParms.N
		targetInd = find(Data.dtarget(trial, :) == 0, 1);
		szInd = find(szList == expParms.setsize(trial), 1);
        Pchoose(trial,:) = circshift(vPDFSz(szInd,:), [0, targetInd-1]);  % shifts the vMises with peak at 1 such that the peak is at targetInd
	end
    Pred = Pchoose;
    likelihood = zeros(expParms.N,1);
    for trial = 1:expParms.N
        likelihood(trial) = Pred(trial, Data.responsegrad(trial));
    end
    output.Pred = Pred;
    output.likelihood = sum(-2*log(likelihood));
end

x = 1; 
