function [ mtarget, breakvar, N] = aggregate(by, target, aggfun)
%[mtarget, breakvar, N] = aggregate(by, target, aggfun)
%aggregates like SPSS. 
%Arguments: by = column vector (or matrix) of break variable(s), target = (column) vector or matrix of
%to-be-aggregated variables, by and target must have same vertical size!
%aggfun = function, provided as function handle!
%Output: breakvar = column vector (or matrix) of aggregated break
%variables; mtarget = aggregated variables

if nargin < 3, aggfun = @mean; end

breakvar = unique(by, 'rows');
N = zeros(1,length(breakvar));
for a = 1:size(breakvar, 1)
    if size(breakvar,2) == 1,   %for the case of a single break variable
        if size(target,2) == 1, selectedrows = target(by==breakvar(a));  %for the case of a single target variable
        else selectedrows = target(by==breakvar(a),:);  
        end
    else
        sel = ones(size(target,1),1);
        for b = 1:size(breakvar,2)
            sel = sel .* (by(:,b)==breakvar(a,b));
        end
        if size(target,2) == 1, selectedrows = target(sel==1); 
        else selectedrows = target(sel==1,:); 
        end
    end
    if size(selectedrows,1) > 1, 
        mtarget(a,:) = aggfun(selectedrows);
        N(a) = size(selectedrows,1);
    else
        mtarget(a,:) = selectedrows;
        N(a) = 1;
    end
end
